/*
 * Copyright (c) 2002-2009, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.document.service;

import fr.paris.lutece.plugins.document.business.Document;
import fr.paris.lutece.plugins.document.business.DocumentHome;
import fr.paris.lutece.plugins.document.business.DocumentType;
import fr.paris.lutece.plugins.document.business.DocumentTypeHome;
import fr.paris.lutece.plugins.document.business.attributes.DocumentAttribute;
import fr.paris.lutece.plugins.document.business.category.Category;
import fr.paris.lutece.plugins.document.business.portlet.DocumentPortletHome;
import fr.paris.lutece.plugins.document.business.spaces.DocumentSpace;
import fr.paris.lutece.plugins.document.business.spaces.DocumentSpaceHome;
import fr.paris.lutece.plugins.document.business.workflow.DocumentAction;
import fr.paris.lutece.plugins.document.business.workflow.DocumentActionHome;
import fr.paris.lutece.plugins.document.service.publishing.PublishingService;
import fr.paris.lutece.plugins.document.service.spaces.DocumentSpacesService;
import fr.paris.lutece.plugins.document.service.IDocumentActionsService;
import fr.paris.lutece.plugins.document.web.DocumentResourceServlet;
import fr.paris.lutece.portal.business.portlet.Portlet;
import fr.paris.lutece.portal.business.user.AdminUser;
import fr.paris.lutece.portal.service.cache.CacheableService;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.portal.PortalService;
import fr.paris.lutece.portal.service.rbac.RBACResource;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.spring.SpringContextService;
import fr.paris.lutece.util.date.DateUtil;
import fr.paris.lutece.util.xml.XmlUtil;

import java.sql.Timestamp;

import java.util.Collection;
import java.util.List;
import java.util.Locale;


/**
 * This Service manages document actions (create, move, delete, validate ...)
 * and notify listeners.
 */
public class DocumentService
{
    private static DocumentService _singleton = new DocumentService(  );
    private static final String TAG_DOCUMENT_ID = "document-id";
    private static final String TAG_DOCUMENT_TITLE = "document-title";
    private static final String TAG_DOCUMENT_SUMMARY = "document-summary";
    private static final String TAG_DOCUMENT_ACCEPT_SITE_COMMENTS = "document-is-commentable";
    private static final String TAG_DOCUMENT_DATE_BEGIN = "document-date-begin";
    private static final String TAG_DOCUMENT_DATE_END = "document-date-end";
    private static final String TAG_DOCUMENT_CATEGORIES = "document-categories";
    private static final String TAG_DOCUMENT_CATEGORY = "category";
    private static final String TAG_CDATA_BEGIN = "<![CDATA[";
    private static final String TAG_CDATA_END = "]]>";
    private static DocumentEventListernersManager _managerEventListeners;

    /** Creates a new instance of DocumentService */
    private DocumentService(  )
    {
        _managerEventListeners = (DocumentEventListernersManager) SpringContextService.getPluginBean( "document",
                "document.documentEventListernersManager" );
    }

    /**
     * Get the unique instance of the service
     *
     * @return The unique instance
     */
    public static DocumentService getInstance(  )
    {
        return _singleton;
    }

    /**
     * Build an XML document that contains document's data.
     *
     * @param document The document
     * @return An XML fragment containing document's data
     */
    String buildXmlContent( Document document )
    {
        StringBuffer sbXml = new StringBuffer(  );
        XmlUtil.beginElement( sbXml, document.getCodeDocumentType(  ) );

        XmlUtil.addElement( sbXml, TAG_DOCUMENT_ID, document.getId(  ) );
        XmlUtil.addElement( sbXml, TAG_DOCUMENT_TITLE, TAG_CDATA_BEGIN + document.getTitle(  ) + TAG_CDATA_END );
        XmlUtil.addElement( sbXml, TAG_DOCUMENT_SUMMARY, TAG_CDATA_BEGIN + document.getSummary(  ) + TAG_CDATA_END );
        XmlUtil.addElement( sbXml, TAG_DOCUMENT_ACCEPT_SITE_COMMENTS, document.getAcceptSiteComments(  ) );
        XmlUtil.addElement( sbXml, TAG_DOCUMENT_DATE_BEGIN,
            DateUtil.getDateString( document.getDateValidityBegin(  ), I18nService.getDefaultLocale(  ) ) );
        XmlUtil.addElement( sbXml, TAG_DOCUMENT_DATE_END,
            DateUtil.getDateString( document.getDateValidityEnd(  ), I18nService.getDefaultLocale(  ) ) );
        XmlUtil.addElement( sbXml, TAG_DOCUMENT_CATEGORIES, buildXmlCategories( document ) );

        DocumentType documentType = DocumentTypeHome.findByPrimaryKey( document.getCodeDocumentType(  ) );

        for ( DocumentAttribute attribute : documentType.getAttributes(  ) )
        {
            DocumentAttribute attributeDocument = getDocumentAttribute( document, attribute.getId(  ) );

            if ( attributeDocument != null )
            {
                AttributeManager manager = AttributeService.getInstance(  )
                                                           .getManager( attributeDocument.getCodeAttributeType(  ) );
                XmlUtil.addElement( sbXml, document.getCodeDocumentType(  ) + "-" + attribute.getCode(  ),
                    manager.getAttributeXmlValue( document, attributeDocument ) );
            }
        }

        XmlUtil.endElement( sbXml, document.getCodeDocumentType(  ) );

        return sbXml.toString(  );
    }

    /**
     * Build  XML content for the categories
     * @param document The document
     * @return An XML fragment containing document's categories
     */
    private String buildXmlCategories( Document document )
    {
        StringBuffer strListCategories = new StringBuffer(  );

        if ( ( document != null ) && ( document.getCategories(  ) != null ) && !document.getCategories(  ).isEmpty(  ) )
        {
            for ( Category category : document.getCategories(  ) )
            {
                XmlUtil.addElement( strListCategories, TAG_DOCUMENT_CATEGORY,
                    TAG_CDATA_BEGIN + category.getName(  ) + TAG_CDATA_END );
            }
        }

        return strListCategories.toString(  );
    }

    /**
     * Change the state of the document
     *
     * @param document The document
     * @param user The user doing the action
     * @param nStateId The new state Id
     * @throws DocumentException raise when error occurs in event or rule
     */
    public void changeDocumentState( Document document, AdminUser user, int nStateId )
        throws DocumentException
    {
        document.setStateId( nStateId );
        DocumentHome.update( document, false );

        notify( document.getId(  ), user, DocumentEvent.DOCUMENT_STATE_CHANGED );
    }

    /**
     * Create a new document
     *
     * @param document The document
     * @param user The user doing the action
     * @throws DocumentException raise when error occurs in event or rule
     */
    public void createDocument( Document document, AdminUser user )
        throws DocumentException
    {
        document.setId( DocumentHome.newPrimaryKey(  ) );
        document.setDateCreation( new Timestamp( new java.util.Date(  ).getTime(  ) ) );
        document.setDateModification( new Timestamp( new java.util.Date(  ).getTime(  ) ) );
        document.setXmlWorkingContent( buildXmlContent( document ) );

        DocumentHome.create( document );

        notify( document.getId(  ), user, DocumentEvent.DOCUMENT_CREATED );
    }

    /**
     * Modify a the content of a document
     *
     * @param document The document
     * @param user The user doing the action
     * @throws DocumentException raise when error occurs in event or rule
     */
    public void modifyDocument( Document document, AdminUser user )
        throws DocumentException
    {
        document.setDateModification( new Timestamp( new java.util.Date(  ).getTime(  ) ) );
        document.setXmlWorkingContent( buildXmlContent( document ) );
        DocumentHome.update( document, true );

        notify( document.getId(  ), user, DocumentEvent.DOCUMENT_CONTENT_MODIFIED );
    }

    /**
     * Validate a document
     * @param nStateId The new state id for a validated document
     * @param document The document
     * @param user The user doing the action
     * @throws DocumentException raise when error occurs in event or rule
     */
    public void validateDocument( Document document, AdminUser user, int nStateId )
        throws DocumentException
    {
    	document.setStateId( nStateId );
        // Copy the working content into the validated content
        document.setXmlValidatedContent( document.getXmlWorkingContent(  ) );
        DocumentHome.update( document, false );
        DocumentHome.validateAttributes( document.getId( ) );
        List<DocumentAttribute> listAttributes = document.getAttributes(  );

        for ( DocumentAttribute attribute : listAttributes )
        {
           DocumentResourceServlet.putInCache( document.getId(  ), attribute.getId(  ) );
        }

        for ( CacheableService cs : PortalService.getCacheableServicesList(  ) )
        {
        	if( cs.getClass().equals(DocumentContentService.class) )
        	{
        		for (int nIdPortlet : DocumentPortletHome.findPortletForDocument(document.getId(  ) ) )
        		{
        			( ( DocumentContentService ) cs ).removeFromCache( Integer.toString( document.getId(  ) ),Integer.toString( nIdPortlet ) );
        		}        		
        	}
        }
        
        notify( document.getId(  ), user, DocumentEvent.DOCUMENT_STATE_CHANGED );
    }

    /**
     * Archive a document
     * @param nStateId The new state id for a validated document
     * @param document The document
     * @param user The user doing the action
     * @throws DocumentException raise when error occurs in event or rule
     */
    public void archiveDocument( Document document, AdminUser user, int nStateId )
        throws DocumentException
    {
        // Get the list of portlets linked with document 
        Collection<Portlet> listPortlet = PublishingService.getInstance(  )
                                                           .getPortletsByDocumentId( Integer.toString( 
                    document.getId(  ) ) );

        for ( Portlet portlet : listPortlet )
        {
            //If document is published, unpublish it
            if ( PublishingService.getInstance(  ).isPublished( document.getId(  ), portlet.getId(  ) ) )
            {
                PublishingService.getInstance(  ).unPublish( document.getId(  ), portlet.getId(  ) );
            }

            //Unassign Document to portlet
            PublishingService.getInstance(  ).unAssign( document.getId(  ), portlet.getId(  ) );
        }

        document.setStateId( nStateId );
        DocumentHome.update( document, false );

        notify( document.getId(  ), user, DocumentEvent.DOCUMENT_STATE_CHANGED );
    }

    /**
     * Move a document from a space to another
     *
     * @param document The document
     * @param user The user doing the action
     * @param nNewSpace The Id of the destination space
     * @throws DocumentException raise when error occurs in event or rule
     */
    public void moveDocument( Document document, AdminUser user, int nNewSpace )
        throws DocumentException
    {
        document.setSpaceId( nNewSpace );
        DocumentHome.update( document, false );

        notify( document.getId(  ), user, DocumentEvent.DOCUMENT_MOVED );
    }

    /**
     * Notify an event to all listeners
     *
     * @param nDocumentId The document Id
     * @param user The user doing the action
     * @param nEventType The type of event
     * @throws DocumentException raise when error occurs in event or rule
     */
    private void notify( int nDocumentId, AdminUser user, int nEventType )
        throws DocumentException
    {
        // Reload document to have all data (ie : state_key !)
        Document document = DocumentHome.findByPrimaryKeyWithoutBinaries( nDocumentId );
        DocumentEvent event = new DocumentEvent( document, user, nEventType );

        _managerEventListeners.notifyListeners( event );
    }

    /**
     * Add to the document all permitted actions according to the current user and
     * using the current locale
     * @param user The current user
     * @param document The document
     * @param locale The Locale
     */
    public void getActions( Document document, Locale locale, AdminUser user )
    {
        List<DocumentAction> listActions = DocumentActionHome.getActionsList( document, locale );
        RBACResource documentType = DocumentTypeHome.findByPrimaryKey( document.getCodeDocumentType(  ) );
        listActions = (List) RBACService.getAuthorizedActionsCollection( listActions, documentType, user );
        
        // Get all beans from the global ApplicationContext
        List<IDocumentActionsService> listActionServices = SpringContextService.getBeansOfType( IDocumentActionsService.class );
        
        // Process all services
        for ( IDocumentActionsService actionService : listActionServices )
        {
        	listActions.addAll( actionService.getActions( document, locale, user) );
        }
        
        document.setActions( listActions );
    }

    public void getPublishedStatus( Document document )
    {
        boolean isPublished = PublishingService.getInstance(  ).isPublished( document.getId(  ) );
        int nPublishedStatus = 0;

        if ( !isPublished )
        {
            nPublishedStatus = 1;
        }

        document.setPublishedStatus( nPublishedStatus );
    }

    /**
     * Build an HTML form for the document creation for a given document type
     *
     * @param strDocumentTypeCode The Document type code
     * @param locale The Locale
     * @param strBaseUrl The base Url
     *
     * @return The HTML form
     */
    public String getCreateForm( String strDocumentTypeCode, Locale locale, String strBaseUrl )
    {
        StringBuffer sbForm = new StringBuffer(  );
        DocumentType documentType = DocumentTypeHome.findByPrimaryKey( strDocumentTypeCode );

        for ( DocumentAttribute attribute : documentType.getAttributes(  ) )
        {
            AttributeManager manager = AttributeService.getInstance(  ).getManager( attribute.getCodeAttributeType(  ) );
            sbForm.append( manager.getCreateFormHtml( attribute, locale, strBaseUrl ) );
        }

        return sbForm.toString(  );
    }

    /**
     * Build an HTML form for the document modification for a given document
     *
     * @param strDocumentId The Id of the document to modify
     * @param locale The Locale
     * @param strBaseUrl The base url
     *
     * @return The HTML form
     */
    public String getModifyForm( String strDocumentId, Locale locale, String strBaseUrl )
    {
        StringBuffer sbForm = new StringBuffer(  );
        int nDocumentId = Integer.parseInt( strDocumentId );
        Document document = DocumentHome.findByPrimaryKey( nDocumentId );
        DocumentType documentType = DocumentTypeHome.findByPrimaryKey( document.getCodeDocumentType(  ) );

        for ( DocumentAttribute attribute : documentType.getAttributes(  ) )
        {
            DocumentAttribute attributeDocument = getDocumentAttribute( document, attribute.getId(  ) );

            if ( attributeDocument != null )
            {
                AttributeManager manager = AttributeService.getInstance(  )
                                                           .getManager( attributeDocument.getCodeAttributeType(  ) );
                sbForm.append( manager.getModifyFormHtml( attributeDocument, document, locale, strBaseUrl ) );
            }
            else
            {
                AttributeManager manager = AttributeService.getInstance(  )
                                                           .getManager( attribute.getCodeAttributeType(  ) );
                sbForm.append( manager.getCreateFormHtml( attribute, locale, strBaseUrl ) );
            }
        }

        return sbForm.toString(  );
    }

    /**
     * Retrieve an attribute of document from its ID
     * @param document The document
     * @param nAttributeId The attribute ID
     * @return The attribute if found, otherwise null
     */
    private DocumentAttribute getDocumentAttribute( Document document, int nAttributeId )
    {
        for ( DocumentAttribute attribute : document.getAttributes(  ) )
        {
            if ( attribute.getId(  ) == nAttributeId )
            {
                return attribute;
            }
        }

        return null;
    }

    /**
     * Check that a given user is allowed to access a document type  for a given permission in a document space specified in parameter
     * If permission is document creation, check if document creation is allowed for the specified space.
     * @param  nIdSpace the id of the document space
     * @param strDocumentTypeId the id of the type document being considered
     * @param strPermission the permission needed
     * @param user the user trying to access the ressource
     *@return true if the user can access the given resource with the given permission, false otherwise
     */
    public boolean isAuthorizedAdminDocument( int nIdSpace, String strDocumentTypeId, String strPermission,
        AdminUser user )
    {
        DocumentSpace documentSpace = DocumentSpaceHome.findByPrimaryKey( nIdSpace );
        boolean bPermissionCreate = strPermission.equals( DocumentTypeResourceIdService.PERMISSION_CREATE )
            ? documentSpace.isDocumentCreationAllowed(  ) : true;

        return DocumentSpacesService.getInstance(  ).isAuthorizedViewByWorkgroup( documentSpace.getId(  ), user ) &&
        DocumentSpacesService.getInstance(  ).isAuthorizedViewByRole( documentSpace.getId(  ), user ) &&
        RBACService.isAuthorized( DocumentType.RESOURCE_TYPE, strDocumentTypeId, strPermission, user ) &&
        bPermissionCreate;
    }
}
